/***************************************************************************
 * Copyright 2014 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.diagnosis.common.model.importer.stages;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import kieker.common.record.controlflow.OperationExecutionRecord;
import kieker.diagnosis.common.domain.OperationCall;
import kieker.diagnosis.common.domain.Trace;

/**
 * Reconstruct traces based on the incoming instances of {@code OperationExecutionRecord}.
 *
 * @author Nils Christian Ehmke
 */
final class LegacyTraceReconstructor extends AbstractStage<OperationExecutionRecord, Trace> {

	private final Map<Long, TraceBuffer> traceBuffers = new HashMap<>();

	@Override
	protected void execute(final OperationExecutionRecord input) {
		this.handleOperationExecutionRecord(input);
	}

	private void handleOperationExecutionRecord(final OperationExecutionRecord input) {
		final long traceID = input.getTraceId();
		if (!this.traceBuffers.containsKey(traceID)) {
			this.traceBuffers.put(traceID, new TraceBuffer(traceID));
		}
		final TraceBuffer traceBuffer = this.traceBuffers.get(traceID);

		traceBuffer.handleEvent(input);
		if (traceBuffer.isTraceComplete()) {
			final Trace trace = traceBuffer.reconstructTrace();
			super.send(trace);
		}
	}

	private static final class TraceBuffer {

		private final List<OperationExecutionRecord> records = new ArrayList<>();
		private final long traceID;
		private boolean traceComplete = false;

		public TraceBuffer(final long traceID) {
			this.traceID = traceID;
		}

		public void handleEvent(final OperationExecutionRecord record) {
			this.records.add(record);

			if ((record.getEoi() == 0) && (record.getEss() == 0)) {
				this.traceComplete = true;
			}
		}

		public Trace reconstructTrace() {
			Collections.sort(this.records, new EOIComparator());

			OperationCall root = null;
			OperationCall header = null;
			int ess = 0;
			for (final OperationExecutionRecord record : this.records) {
				final OperationCall newCall = new OperationCall(record.getHostname(), this.extractComponent(record.getOperationSignature()), record.getOperationSignature(),
						this.traceID);
				newCall.setDuration(record.getTout() - record.getTin());

				if ((record.getEss() <= ess) && (ess != 0)) {
					header = header.getParent();
				}

				if (root == null) {
					root = newCall;
				} else {
					header.addChild(newCall);
				}
				header = newCall;
				ess = record.getEss();
			}

			return new Trace(root, this.traceID);
		}

		private String extractComponent(final String operationSignature) {
			return operationSignature.replaceFirst("\\.\\w*\\(.*", "");
		}

		public boolean isTraceComplete() {
			return this.traceComplete;
		}

		private static final class EOIComparator implements Comparator<OperationExecutionRecord>, Serializable {

			private static final long serialVersionUID = 1L;

			@Override
			public int compare(final OperationExecutionRecord o1, final OperationExecutionRecord o2) {
				return Long.compare(o1.getEoi(), o2.getEoi());
			}

		}

	}

}
